@@ -1,12 +1,7 @@
 {-# LANGUAGE CPP #-}
 
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
-#if defined(MIN_VERSION_hashable) || __GLASGOW_HASKELL__ == 702 \
-                                  || __GLASGOW_HASKELL__ >= 708
 {-# LANGUAGE Trustworthy #-}
-#else
-{-# LANGUAGE Safe #-}
-#endif
 #endif
 
 #ifdef __GLASGOW_HASKELL__
@@ -171,6 +166,14 @@ import qualified GHC.Exts as Exts
 import GHC.Generics
 #endif
 
+#ifdef MIN_VERSION_template_haskell
+import qualified Language.Haskell.TH.Syntax as TH
+#ifndef CURRENT_PACKAGE_KEY
+import Data.Version (showVersion)
+import Paths_semigroups (version)
+#endif
+#endif
+
 infixr 5 :|, <|
 
 data NonEmpty a = a :| [a] deriving
@@ -188,11 +191,7 @@ data NonEmpty a = a :| [a] deriving
 
 #ifdef MIN_VERSION_hashable
 instance Hashable a => Hashable (NonEmpty a) where
-#if MIN_VERSION_hashable(1,2,0)
   hashWithSalt p (a :| as) = p `hashWithSalt` a `hashWithSalt` as
-#else
-  hash (a :| as) = hash a `combine` hash as
-#endif
 #endif
 
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
@@ -205,23 +204,23 @@ instance Exts.IsList (NonEmpty a) where
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702 && __GLASGOW_HASKELL__ < 706
 instance Generic1 NonEmpty where
   type Rep1 NonEmpty
-    = D1 D1NonEmpty
-        (C1 C1_0NonEmpty
+    = D1 D1'NonEmpty
+        (C1 C1'_0NonEmpty
              (S1 NoSelector Par1
           :*: S1 NoSelector (Rec1 [])))
   from1 (h :| t) = M1 (M1 (M1 (Par1 h) :*: M1 (Rec1 t)))
   to1 (M1 (M1 (M1 h :*: M1 t))) = unPar1 h :| unRec1 t
 
-instance Datatype D1NonEmpty where
+instance Datatype D1'NonEmpty where
   datatypeName _ = "NonEmpty"
   moduleName   _ = "Data.List.NonEmpty"
 
-instance Constructor C1_0NonEmpty where
+instance Constructor C1'_0NonEmpty where
   conName   _ = ":|"
   conFixity _ = Infix RightAssociative 5
 
-data D1NonEmpty
-data C1_0NonEmpty
+data D1'NonEmpty
+data C1'_0NonEmpty
 #endif
 
 #ifdef MIN_VERSION_deepseq
@@ -346,6 +345,24 @@ instance Foldable NonEmpty where
   toList = toList
 #endif
 
+#ifdef MIN_VERSION_template_haskell
+-- | @since 0.19
+instance TH.Lift a => TH.Lift (NonEmpty a) where
+    lift (x :| xs) = do
+        x' <- TH.lift x
+        xs' <- TH.lift xs
+        return $ TH.InfixE (Just x') (TH.ConE neConsName) (Just xs')
+      where
+        neConsName = TH.mkNameG_d semigroupsPackageKey "Data.List.NonEmpty" ":|"
+
+semigroupsPackageKey         :: String
+#ifdef CURRENT_PACKAGE_KEY
+semigroupsPackageKey          = CURRENT_PACKAGE_KEY
+#else
+semigroupsPackageKey          = "semigroups-" ++ showVersion version
+#endif
+#endif
+
 -- | Extract the first element of the stream.
 head :: NonEmpty a -> a
 head ~(a :| _) = a
